package bluetooth;

import java.util.Vector;
import javax.bluetooth.*;
import java.io.*; 
import javax.microedition.io.*; 

public class NWSerial implements DiscoveryListener
{
	private String LocalAddress;
	private String LocalName;
	
	private Vector ListDevices = new Vector();
	private Vector ListServices = new Vector();
	private DiscoveryAgent agent;
	private boolean IDeviceSearchFinished;
	private boolean IServiceSearchFinished;
	
	private StreamConnection SPPConnection;
	private DataInputStream SerialStreamIn;
	private DataOutputStream SerialStreamOut;
	
	private String ServiceURL="";
	private boolean Connected=false;
	
	NWSerial()
	{
		
	}
	
	public int Init()
	{
		try
		{
			LocalDevice local = LocalDevice.getLocalDevice();
			// retrieve the Bluetooth address of the local device
			LocalAddress = local.getBluetoothAddress();
			// retrieve the name of the local Bluetooth device
			LocalName = local.getFriendlyName();
			// retrieve the discovery agent 
			agent = local.getDiscoveryAgent();
		}
		catch(BluetoothStateException Error)
		{
			return 0;
		}
		return 1;
	}
	
	/* devices */
	public int SearchForDevices()
	{
		IDeviceSearchFinished=false;
		ListDevices.removeAllElements();
		try
		{
			// place the device in inquiry mode
			boolean complete = agent.startInquiry(DiscoveryAgent.GIAC, this);
			if(!complete)
				return 0;
			agent.retrieveDevices(DiscoveryAgent.CACHED);
		}
		catch(BluetoothStateException Error)
		{
			return 0;
		}
		IDeviceSearchFinished=false;
		return 1;
	}
	
	public void deviceDiscovered(RemoteDevice NewRemoteDevice, DeviceClass NewDeviceClass)
	{
		ListDevices.addElement(NewRemoteDevice);
	}
	
	// device scan complete
	public void inquiryCompleted(int discoveryType)
	{
		IDeviceSearchFinished=true;
	}
	
	/* services */
	public int SearchForServices(RemoteDevice SingleDevice)
	{
		ListServices.removeAllElements();
		UUID[] uuidList = new UUID[1];
		uuidList[0]=new UUID(0x0003);	// 0x0003 = RFCOMM or UUID? no idea ^^
		int[] attrSet = { 0x0000, 0x0001, 0x0004, 0x0100 };
		try
		{
			agent.searchServices(attrSet, uuidList, SingleDevice, this);
		}
		catch(BluetoothStateException Error)
		{
			return 0;
		}
		IServiceSearchFinished=false;
		return 1;
	}
	
	public void servicesDiscovered(int arg0, ServiceRecord[] NewSR)
	{
		for(int i=0; i<NewSR.length; i++)
		{
				ListServices.addElement(NewSR[i]);
		}
	}
	
	// service scan complete
	public void serviceSearchCompleted(int arg0, int arg1)
	{
		IServiceSearchFinished=true;
	}
	
	
	/* --- public --- */
	/* devices */
	public boolean DeviceSearchFinished()
	{
		return IDeviceSearchFinished;
	}
	
	public int GetDeviceCout()
	{
		return ListDevices.size();
	}
	
	public Vector GetDevices()
	{
		return ListDevices;
	}
	
	/* services */
	public boolean ServiceSearchFinished()
	{
		return IServiceSearchFinished;
	}
	
	public int GetServiceCount()
	{
		return ListServices.size();
	}
	
	public Vector GetServices()
	{
		return ListServices;
	}
	
	/* local */
	public String GetLocalAddress()
	{
		return LocalAddress;
	}
	
	public String GetLocalName()
	{
		return LocalName;
	}
	
	/* misc */
	public int Connect(String NewServiceURL)
	{
		ServiceURL=NewServiceURL;
		try
		{
			SPPConnection = (StreamConnection) Connector.open(ServiceURL);
			SerialStreamIn = SPPConnection.openDataInputStream();
			SerialStreamOut = SPPConnection.openDataOutputStream();
		}
		catch(IOException Error)
		{
			return 0;
		}
		Connected=true;
		return 1;
	}
	
	public int Disconnect()
	{
		Connected=false;
		try
		{
			if(SerialStreamIn!=null)
				SerialStreamIn.close();
			if(SerialStreamOut!=null)
				SerialStreamOut.close();
			if(SPPConnection!=null)
				SPPConnection.close();
		}
		catch(IOException Error)
		{
			return 0;
		}
		return 1;
	}
	
	public int Reconnect()
	{
		Connected=false;
		if(ServiceURL=="")
			return -1;
		if(Disconnect()==0)
			return 0;
		return Connect(ServiceURL);
	}
	
	/* data in/out */
	public int Available()
	{
		if(!Connected)
			return 0;
		int BufferSize=0;
		try
		{
			BufferSize=SerialStreamIn.available();
		}
		catch(IOException Error)
		{
			BufferSize=-1;
		}
		return BufferSize;
	}
	
	public int GetData(byte[] Data)
	{
		int ByteRead;
		try
		{
			ByteRead=SerialStreamIn.available();
			if(ByteRead==0)
				return 0;
			if(ByteRead>Data.length)
				ByteRead=Data.length;
			SerialStreamIn.read(Data, 0, ByteRead);
		}
		catch (IOException Error)
		{
			return -1;
		}
		return ByteRead;
	}
	
	public int SendData(byte[] Data, int Size)
	{
		try
		{
			SerialStreamOut.write(Data, 0, Size);
			SerialStreamOut.flush();
		}
		catch (IOException Error)
		{
			return 0;
		}
		return 1;
	}
}
